#ifdef VIRTUAL_REGEX
#include <ctype.h>
#include <regex.h>
#endif
#include <mysql/mysql.h>
#include "auto_break.h"
#include "control.h"
#include "fmt.h"
#include "mysql_queries.h"
#include "stralloc.h"

extern int connect_mysql();

extern MYSQL dbh, *mysql;
extern MYSQL_RES *result;
extern stralloc envnoathost;
extern char strnum3[FMT_ULONG];

extern MYSQL_ROW do_query(stralloc *query);

/* map a virtual domain to a user-ext stralloc */
int vdoms_mysql(char *addr, int len, stralloc *ret) {
  char *host;
  int at, num;
  MYSQL_ROW row;
#ifdef VIRTUAL_REGEX
  int re_offsets[45];
  regex_t re;
  regmatch_t matches[sizeof(re_offsets) / sizeof(int)];
  size_t size;
  int r, i;
  char c;
  int size_offsets = sizeof(re_offsets) / sizeof(int);
#endif
  stralloc qmail_mysql_query = { 0 };
  stralloc real_ext = { 0 };
  stralloc real_addr = { 0 };
  stralloc real_host = { 0 };

  /* stralloc we're passed is unterminated */
  addr[len] = '\0';
/*tcplog("vdoms_mysql: ", addr, "\n");*/
  num = len;
  host = addr + len;
  while (*host != '@') {
    host--;
    num--;
    if (host == addr) break;
  }
  at = num;

  /* let's lose the @ we found */
  host++;
  if (! stralloc_ready(&real_addr, 2 * str_len(addr) + 1)) return -1;
  mysql_escape_string(real_addr.s, addr, at);
  if (! stralloc_ready(&real_host, 2 * str_len(host) + 1)) {
    stralloc_free(&real_addr);
    return -1;
  }
  mysql_escape_string(real_host.s, host, str_len(host));

#define ACK {\
  stralloc_free(&real_addr);\
  stralloc_free(&real_host);\
  stralloc_free(&qmail_mysql_query);\
  return -1;\
}
  if (connect_mysql() < 1) ACK

  len += real_addr.len + real_host.len + str_len(VIRTUAL) + 2;
  if (! stralloc_ready(&qmail_mysql_query, len)) ACK
  if (! stralloc_cats(&qmail_mysql_query, VIRTUAL1)) ACK
  if (! stralloc_cats(&qmail_mysql_query, real_addr.s)) ACK
  if (! stralloc_cats(&qmail_mysql_query, VIRTUAL2)) ACK
  if (! stralloc_cats(&qmail_mysql_query, real_host.s)) ACK
  if (! stralloc_cats(&qmail_mysql_query, VIRTUAL3)) ACK
  if (! stralloc_0(&qmail_mysql_query)) ACK

#ifndef O_NOT_LOG
  log3("query: ", qmail_mysql_query.s, ";\n");
#endif
  row = do_query(&qmail_mysql_query);

  /* no row doesn't mean an error */
  if ((int) row < 1) return (int) row;

  /* run database sanity check: username may not be blank */
  if (! *(row[0])) {
    log3("MySQL misconfiguration: username for host '", real_host.s, "' is blank in virtual table!\n");
    stralloc_free(&real_addr);
    stralloc_free(&real_host);
    mysql_free_result(result);
    return 0;
  }

#ifdef VIRTUAL_REGEX
#define YOW {\
  regfree(&re);\
  stralloc_free(&real_addr);\
  stralloc_free(&real_host);\
  mysql_free_result(result);\
  return -1;\
}

  r = regcomp(&re, row[2], REG_EXTENDED | REG_ICASE);
  if (r) {
    log3("Compilation of regular expression /", row[2], "/ failed for ");
    log2(host, "\n");
    YOW
  }

  r = regexec(&re, real_addr.s, size_offsets, matches, 0);
  if (r) {
    log3("Failed regex: ", real_addr.s, " =~ /");
    log2(row[2], "/ for host ");
    log2(host, "\n");
    YOW
  }

  for (i = 0; (int) *(row[1] + i); i++) {
    c = *(row[1] + i);
    /* backsalsh could be a backreference */
    if (c == '\\') {
      c = *(row[1] + ++i);
      /* it's a number so let's parse it */
      if (c >= '1' && c <= '9') {
        r = (c - '0');
        c = *(row[1] + ++i);
        /* another number: parse that too = shouldn't happen really... */
        if (c >= '1' && c <= '9') {
          r *= 10;
          r += (c - '0');
        }
        else i--;
        if (matches[r].rm_so > -1) {
          if (! stralloc_readyplus(&real_ext, (int) matches[r].rm_eo - (int) matches[r].rm_so)) YOW
          if (! stralloc_catb(&real_ext, real_addr.s + (int) matches[r].rm_so, (int) matches[r].rm_eo - (int) matches[r].rm_so)) YOW
        }
        else {
          log3("Failed backreference: ", real_addr.s, " =~ /");
          log2(row[2], "/ for host ");
          log2(host, " - not enough brackets!\n");
          YOW
        }
      }
      else {
        if (! stralloc_readyplus(&real_ext, 1)) YOW
        if (! stralloc_catb(&real_ext, row[1] + i, 1)) YOW
      }
    }
    /* & means replace the whole match */
    else if (c == '&') {
      if (! stralloc_readyplus(&real_ext, real_addr.len)) YOW
      if (! stralloc_cats(&real_ext, real_addr.s)) YOW
    }
    /* skip straight to the next metacharacter or the end */
    else {
      for (r = 0; *(row[1] + i + r) != '\\' && *(row[1] + i + r) != '&' && *(row[1] + i + r) != '\0'; r++);
      if (r) {
        if (! stralloc_readyplus(&real_ext, r + 1)) YOW
        if (! stralloc_catb(&real_ext, row[1] + i, r)) YOW
        i += r - 1;
      }
    }
  }
  if (! stralloc_0(&real_ext)) YOW

#else
#define YOW {\
  stralloc_free(&real_addr);\
  stralloc_free(&real_host);\
  mysql_free_result(result);\
  return -1;\
}

  if (! stralloc_ready(&real_ext, str_len(row[1]) + 1)) YOW
  if (! stralloc_cats(&real_ext, row[1])) YOW
  if (! stralloc_0(&real_ext)) YOW
#endif

#ifdef MAGIC_FORWARDING
  if (! str_diff(row[0], MAGIC_FORWARD_USER)) {
    if (! stralloc_cats(ret, real_ext.s)) YOW
    if (! stralloc_0(ret)) YOW
    mysql_free_result(result);
    stralloc_free(&real_addr);
    stralloc_free(&real_host);
#ifdef MAGIC_LOGGING
    log2("magic forward: msg ", strnum3);
    log2(" for ", addr);
    log3(" to ", real_ext.s, "\n");
#endif
    return 2;
  }
#endif
  if (! stralloc_cats(ret, row[0])) YOW  /* username */
  if (str_len(real_ext.s)) {
    if (! stralloc_append(ret, auto_break)) YOW
    if (*(real_ext.s) != '@') { if (! stralloc_cats(ret, real_ext.s)) YOW }
    else if (! stralloc_cats(ret, real_addr.s)) YOW  /* ext */
  }
  if (! stralloc_append(ret, "@")) return -1;
  if (! stralloc_cats(ret, real_host.s)) return -1;
  if (! stralloc_0(ret)) return -1;
  mysql_free_result(result);
  stralloc_free(&real_addr);
  stralloc_free(&real_host);
  return 1;
}
